@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: buildhash.c,v 1.70 2001/07/25 21:51:45 geoff Exp $";
+    "$Id: buildhash.c,v 1.74 2005/04/28 00:26:06 geoff Exp $";
 #endif
 
 #define MAIN
@@ -10,7 +10,7 @@ static char Rcs_Id[] =
  *
  * Pace Willisson, 1983
  *
- * Copyright 1992, 1993, 1999, 2001, Geoff Kuenning, Claremont, CA
+ * Copyright 1992, 1993, 1999, 2001, 2005, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,16 +26,8 @@ static char Rcs_Id[] =
  *    such.  Binary redistributions based on modified source code
  *    must be clearly marked as modified versions in the documentation
  *    and/or other materials provided with the distribution.
- * 4. Any web site or other electronic service that offers ispell for
- *    download or other electronic transfer as a separate entity, in
- *    either source or binary form, must also include a prominent statement
- *    indicating that information about ispell can be obtained from the
- *    following Web site URL:
- *	http://fmg-www.cs.ucla.edu/geoff/ispell.html
- *    If the offering service supports hyperlinks, the aforementioned
- *    Web site must also be offered as a hyperlink.  Condition #4 does
- *    not apply if ispell is offered only as part of a larger, aggregated
- *    product such as a word processor or packaged operating system.
+ * 4. The code that causes the 'ispell -v' command to display a prominent
+ *    link to the official ispell Web site may not be removed.
  * 5. The name of Geoff Kuenning may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
@@ -55,6 +47,27 @@ static char Rcs_Id[] =
 
 /*
  * $Log: buildhash.c,v $
+ * Revision 1.74  2005/04/28 00:26:06  geoff
+ * Re-count the dictionary file every time, rather than depending on a
+ * file to hold the correct count.  This is necessary because the changes
+ * for MS-DOS support caused us to generate non-unique count-file names,
+ * which in turn could cause buildhash to fail.  There really isn't a
+ * need for count files any more; they were only a performance
+ * improvement and modern computers are fast enough that it doesn't
+ * matter.
+ *
+ * Revision 1.73  2005/04/14 21:25:52  geoff
+ * Declare ints-that-hold-pointers as unsigned, just for safety.
+ *
+ * Revision 1.72  2005/04/14 14:38:23  geoff
+ * Update license and copyright.  Fix a count-file-naming bug introduced
+ * in a recent delta.  Regenerate the count file if it has the same mtime
+ * as the dictionary (otherwise you can have bugs on fast machines).  Fix
+ * some small type bugs.
+ *
+ * Revision 1.71  2001/09/06 00:30:28  geoff
+ * Many changes from Eli Zaretskii to support DJGPP compilation.
+ *
  * Revision 1.70  2001/07/25 21:51:45  geoff
  * Minor license update.
  *
@@ -108,18 +121,15 @@ VOID *		myrealloc P ((VOID * ptr, unsigned int size,
 		  unsigned int oldsize));
 void		myfree P ((VOID * ptr));
 static void	readdict P ((void));
-static void	newcount P ((void));
+static unsigned int
+		newcount P ((void));
 
 #define NSTAT	100		/* Size probe-statistics table */
 
-struct stat	dstat;		/* Result of stat-ing dict file */
-struct stat	cstat;		/* Result of stat-ing count file */
-
 char *		Dfile;		/* Name of dictionary file */
 char *		Hfile;		/* Name of hash (output) file */
 char *		Lfile;		/* Name of language file */
 
-char		Cfile[MAXPATHLEN]; /* Name of count file */
 char		Sfile[MAXPATHLEN]; /* Name of statistics file */
 
 static int silent = 0;		/* NZ to suppress count reports */
@@ -129,7 +139,7 @@ int main (argc, argv)
     char *	argv[];
     {
     int		avg;
-    FILE *	countf;
+    char *	lastdot;
     FILE *	statf;
     int		stats[NSTAT];
     unsigned int i;
@@ -164,28 +174,29 @@ int main (argc, argv)
     if (yyparse ())			/* Parse the language tables */
 	exit (1);
 
-    (void) sprintf (Cfile, "%s%s", Dfile, COUNTSUFFIX);
-    (void) sprintf (Sfile, "%s%s", Dfile, STATSUFFIX);
+    strcpy (Sfile, Dfile);
+    lastdot = rindex (Sfile, '.');
+    if (lastdot != NULL)
+	*lastdot = '\0';
+    strcat (Sfile, STATSUFFIX);
+#ifdef MSDOS
+    /*
+    ** MS-DOS doesn't allow more than one dot in the filename part.
+    ** If we have more than that, convert all the dots but the last into
+    ** underscores.  The OS will truncate excess characters beyond 8+3.
+    */
+    lastdot = rindex (Sfile, '.');
+    if (lastdot != NULL  &&  lastdot > Sfile)
+	{
+	while  (--lastdot >= Sfile)
+	    {
+	    if (*lastdot == '.')
+		*lastdot = '_';
+	    }
+	}
+#endif /* MSDOS */
 
-    if (stat (Dfile, &dstat) < 0)
-	{
-	(void) fprintf (stderr, BHASH_C_NO_DICT, Dfile);
-	    exit (1);
-	}
-    if (stat (Cfile, &cstat) < 0 || dstat.st_mtime > cstat.st_mtime)
-	newcount ();
-
-    if ((countf = fopen (Cfile, "r")) == NULL)
-	{
-	(void) fprintf (stderr, BHASH_C_NO_COUNT);
-	exit (1);
-	}
-    hashsize = 0;
-    if (fscanf (countf, "%d", &hashsize) != 1  ||  fclose (countf) == EOF)
-	{
-	(void) fprintf (stderr, BHASH_C_BAD_COUNT);
-	exit (1);
-	}
+    hashsize = newcount ();
     if (hashsize == 0)
 	{
 	(void) fprintf (stderr, BHASH_C_ZERO_COUNT);
@@ -195,7 +206,7 @@ int main (argc, argv)
 
     if ((statf = fopen (Sfile, "w")) == NULL)
 	{
-	(void) fprintf (stderr, CANT_CREATE, Sfile);
+	(void) fprintf (stderr, CANT_CREATE, Sfile, MAYBE_CR (stderr));
 	exit (1);
 	}
 
@@ -237,7 +248,7 @@ static void output ()
     {
     register FILE *		houtfile;
     register struct dent *	dp;
-    int				strptr;
+    unsigned long		strptr;
     int				n;
     unsigned int		i;
     int				maxplen;
@@ -246,7 +257,7 @@ static void output ()
 
     if ((houtfile = fopen (Hfile, "wb")) == NULL)
 	{
-	(void) fprintf (stderr, CANT_CREATE, Hfile);
+	(void) fprintf (stderr, CANT_CREATE, Hfile, MAYBE_CR (stderr));
 	return;
 	}
     hashheader.stringsize = 0;
@@ -257,8 +268,10 @@ static void output ()
     /*
     ** Put out the strings from the flags table.  This code assumes that
     ** the size of the hash header is a multiple of the size of ichar_t,
-    ** and that any integer can be converted to an (ichar_t *) and back
-    ** without damage.
+    ** and that any long can be converted to an (ichar_t *) and back
+    ** without damage (or, more accurately, without damaging those
+    ** low-order bits necessary to represent the largest offset in the
+    ** string table).
     */
     maxslen = 0;
     for (i = 0, fentry = sflaglist;  i < numsflags;  i++, fentry++)
@@ -362,7 +375,7 @@ static void output ()
 	{
 	if (dp->next != 0)
 	    {
-	    int		x;
+	    unsigned long	x;
 	    x = dp->next - hashtbl;
 	    dp->next = (struct dent *)x;
 	    }
@@ -624,12 +637,13 @@ static void readdict ()
     (void) fclose (dictf);
     }
 
-static void newcount ()
+static unsigned int newcount ()
     {
     unsigned char	buf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
+    register unsigned int
+			count;
     ichar_t		ibuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
     register FILE *	d;
-    register int	i;
     ichar_t		lastibuf[sizeof ibuf / sizeof (ichar_t)];
     int			headercounted;
     int			followcase;
@@ -646,11 +660,11 @@ static void newcount ()
 
     headercounted = 0;
     lastibuf[0] = 0;
-    for (i = 0;  fgets ((char *) buf, sizeof buf, d);  )
+    for (count = 0;  fgets ((char *) buf, sizeof buf, d);  )
 	{
-	if ((++i % 1000) == 0  &&  !silent)
+	if ((++count % 1000) == 0  &&  !silent)
 	    {
-	    (void) fprintf (stderr, "%d ", i);
+	    (void) fprintf (stderr, "%d ", count);
 	    (void) fflush (stdout);
 	    }
 	cp = index ((char *) buf, hashheader.flagmarker);
@@ -665,9 +679,9 @@ static void newcount ()
 	else if (!headercounted)
 	    {
 	    /* First duplicate will take two entries */
-	    if ((++i % 1000) == 0  &&  !silent)
+	    if ((++count % 1000) == 0  &&  !silent)
 		{
-		(void) fprintf (stderr, "%d ", i);
+		(void) fprintf (stderr, "%d ", count);
 		(void) fflush (stdout);
 		}
 	    headercounted = 1;
@@ -675,9 +689,9 @@ static void newcount ()
 	if (!headercounted  &&  followcase)
 	    {
 	    /* It's followcase and the first entry -- count again */
-	    if ((++i % 1000) == 0  &&  !silent)
+	    if ((++count % 1000) == 0  &&  !silent)
 		{
-		(void) fprintf (stderr, "%d ", i);
+		(void) fprintf (stderr, "%d ", count);
 		(void) fflush (stdout);
 		}
 	    headercounted = 1;
@@ -686,12 +700,7 @@ static void newcount ()
 	}
     (void) fclose (d);
     if (!silent)
-	(void) fprintf (stderr, BHASH_C_WORD_COUNT, i);
-    if ((d = fopen (Cfile, "w")) == NULL)
-	{
-	(void) fprintf (stderr, CANT_CREATE, Cfile);
-	exit (1);
-	}
-    (void) fprintf (d, "%d\n", i);
-    (void) fclose (d);
+	(void) fprintf (stderr, BHASH_C_WORD_COUNT, count);
+
+    return count;
     }
