@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: correct.c,v 1.77 2001/07/25 21:51:47 geoff Exp $";
+    "$Id: correct.c,v 1.82 2005/04/28 14:46:51 geoff Exp $";
 #endif
 
 /*
@@ -11,7 +11,7 @@ static char Rcs_Id[] =
  *
  * Copyright (c), 1983, by Pace Willisson
  *
- * Copyright 1992, 1993, 1999, 2001, Geoff Kuenning, Claremont, CA
+ * Copyright 1992, 1993, 1999, 2001, 2005, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,16 +27,8 @@ static char Rcs_Id[] =
  *    such.  Binary redistributions based on modified source code
  *    must be clearly marked as modified versions in the documentation
  *    and/or other materials provided with the distribution.
- * 4. Any web site or other electronic service that offers ispell for
- *    download or other electronic transfer as a separate entity, in
- *    either source or binary form, must also include a prominent statement
- *    indicating that information about ispell can be obtained from the
- *    following Web site URL:
- *	http://fmg-www.cs.ucla.edu/geoff/ispell.html
- *    If the offering service supports hyperlinks, the aforementioned
- *    Web site must also be offered as a hyperlink.  Condition #4 does
- *    not apply if ispell is offered only as part of a larger, aggregated
- *    product such as a word processor or packaged operating system.
+ * 4. The code that causes the 'ispell -v' command to display a prominent
+ *    link to the official ispell Web site may not be removed.
  * 5. The name of Geoff Kuenning may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
@@ -56,6 +48,27 @@ static char Rcs_Id[] =
 
 /*
  * $Log: correct.c,v $
+ * Revision 1.82  2005/04/28 14:46:51  geoff
+ * Add code to log all corrections and replacements.
+ *
+ * Revision 1.81  2005/04/20 23:16:32  geoff
+ * Add inpossibilities, and use it to deal with the case where uppercase
+ * SS in German causes ambiguities.  Rename some variables to make them
+ * more meaningful.
+ *
+ * Revision 1.80  2005/04/14 14:38:23  geoff
+ * Update license.  Use i-prefixed names for terminal access functions to
+ * prevent conflicts with libraries.  Fix a bug that caused a hang when
+ * using external deformatters.
+ *
+ * Revision 1.79  2001/09/06 00:40:44  geoff
+ * In ask mode, when extending long lines to ensure that a word hasn't
+ * been broken, remember to do the extension inside both the context and
+ * the filtered buffer.
+ *
+ * Revision 1.78  2001/09/06 00:30:29  geoff
+ * Many changes from Eli Zaretskii to support DJGPP compilation.
+ *
  * Revision 1.77  2001/07/25 21:51:47  geoff
  * Minor license update.
  *
@@ -209,6 +222,7 @@ static void	inserttoken P ((unsigned char * buf, unsigned char * start,
 static int	posscmp P ((unsigned char * a, unsigned char * b));
 int		casecmp P ((unsigned char * a, unsigned char * b, int canonical));
 void		makepossibilities P ((ichar_t * word));
+int		inpossibilities P ((unsigned char * ctok));
 static int	insert P ((ichar_t * word));
 static void	wrongcapital P ((ichar_t * word));
 static void	wrongletter P ((ichar_t * word));
@@ -231,7 +245,7 @@ static void	save_root_cap P ((ichar_t * word, ichar_t * pattern,
 		  struct flagent * sufent,
 		  ichar_t savearea[MAX_CAPS][INPUTWORDLEN + MAXAFFIXLEN],
 		  int * nsaved));
-static char *	getline P ((char * buf));
+static char *	getline P ((char * buf, int bufsize));
 void		askmode P ((void));
 void		copyout P ((unsigned char ** cc, int cnt));
 static void	lookharder P ((unsigned char * string));
@@ -249,41 +263,45 @@ void givehelp (interactive)
 
     if (interactive)
 	{
-	erase ();
+	ierase ();
 	helpout = stdout;
 	}
     else
 	helpout = stderr;
 
-    (void) fprintf (helpout, CORR_C_HELP_1);
-    (void) fprintf (helpout, CORR_C_HELP_2);
-    (void) fprintf (helpout, CORR_C_HELP_3);
-    (void) fprintf (helpout, CORR_C_HELP_4);
-    (void) fprintf (helpout, CORR_C_HELP_5);
-    (void) fprintf (helpout, CORR_C_HELP_6);
-    (void) fprintf (helpout, CORR_C_HELP_7);
-    (void) fprintf (helpout, CORR_C_HELP_8);
-    (void) fprintf (helpout, CORR_C_HELP_9);
+    (void) fprintf (helpout, CORR_C_HELP_1, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_2, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_3, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_4, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_5, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_6, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_7, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_8, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_9, MAYBE_CR (helpout));
 
-    (void) fprintf (helpout, CORR_C_HELP_COMMANDS);
+    (void) fprintf (helpout, CORR_C_HELP_COMMANDS, MAYBE_CR (helpout),
+      MAYBE_CR (helpout), MAYBE_CR (helpout));
 
-    (void) fprintf (helpout, CORR_C_HELP_R_CMD);
-    (void) fprintf (helpout, CORR_C_HELP_BLANK);
-    (void) fprintf (helpout, CORR_C_HELP_A_CMD);
-    (void) fprintf (helpout, CORR_C_HELP_I_CMD);
-    (void) fprintf (helpout, CORR_C_HELP_U_CMD);
-    (void) fprintf (helpout, CORR_C_HELP_0_CMD);
-    (void) fprintf (helpout, CORR_C_HELP_L_CMD);
-    (void) fprintf (helpout, CORR_C_HELP_X_CMD);
-    (void) fprintf (helpout, CORR_C_HELP_Q_CMD);
-    (void) fprintf (helpout, CORR_C_HELP_BANG);
-    (void) fprintf (helpout, CORR_C_HELP_REDRAW);
-    (void) fprintf (helpout, CORR_C_HELP_SUSPEND);
-    (void) fprintf (helpout, CORR_C_HELP_HELP);
+    (void) fprintf (helpout, CORR_C_HELP_R_CMD, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_BLANK, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_A_CMD, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_I_CMD, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_U_CMD, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_0_CMD, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_L_CMD, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_X_CMD, MAYBE_CR (helpout),
+      MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_Q_CMD, MAYBE_CR (helpout),
+      MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_BANG, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_REDRAW, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_SUSPEND, MAYBE_CR (helpout));
+    (void) fprintf (helpout, CORR_C_HELP_HELP, MAYBE_CR (helpout));
 
     if (interactive)
 	{
-	(void) fprintf (helpout, "\r\n\r\n");
+	(void) fprintf (helpout, "\r\n");
+	imove (li - 1, 0);  /* bottom line, no matter what screen size is */
 	(void) fprintf (helpout, CORR_C_HELP_TYPE_SPACE);
 	(void) fflush (helpout);
 #ifdef COMMANDFORSPACE
@@ -332,12 +350,14 @@ void checkfile ()
 	if (fgets ((char *) filteredbuf, sizeof filteredbuf / 2, infile)
 	  == NULL)
 	    {
-	    if (sourcefile == NULL)
-		break;
-	    while (fgets ((char *) contextbufs[0], sizeof contextbufs[0],
-		sourcefile)
-	      != NULL)
-		(void) fputs ((char *) contextbufs[0], outfile);
+	    if (sourcefile != NULL)
+		{
+		while (fgets ((char *) contextbufs[0], sizeof contextbufs[0],
+		    sourcefile)
+		  != NULL)
+		    (void) fputs ((char *) contextbufs[0], outfile);
+		}
+	    break;
 	    }
 	/*
 	 * If we didn't read to end-of-line, we may have ended the
@@ -372,7 +392,9 @@ void checkfile ()
 	    {
 	    if (fread (contextbufs[0], 1, bufsize, sourcefile) != bufsize)
 		{
-		(void) fprintf (stderr, CORR_C_SHORT_SOURCE);
+		(void) fprintf (stderr, CORR_C_SHORT_SOURCE,
+		  MAYBE_CR (stderr));
+		(void) sleep ((unsigned) 2);
 		xflag = 0;		/* Preserve file backup */
 		break;
 		}
@@ -407,7 +429,11 @@ checkagain:
     if (good (itok, 0, 0, 0, 0)  ||  compoundgood (itok, 0))
 	return;
 
-    erase ();
+    makepossibilities (itok);
+    if (inpossibilities (ctok))	/* Kludge for German and similar languages */
+	return;
+
+    ierase ();
     (void) printf ("    %s", (char *) ctok);
     if (currentfile)
 	(void) printf (CORR_C_FILE_LABEL, currentfile);
@@ -415,8 +441,6 @@ checkagain:
 	(void) printf (" %s", CORR_C_READONLY);
     (void) printf ("\r\n\r\n");
 
-    makepossibilities (itok);
-
     /*
      * Make sure we have enough room on the screen to hold the
      * possibilities.  Reduce the list if necessary.  co / (maxposslen + 8)
@@ -441,9 +465,9 @@ checkagain:
     for (i = 0; i < pcount; i++)
 	{
 #ifdef BOTTOMCONTEXT
-	move (2 + (i % col_ht), (maxposslen + 8) * (i / col_ht));
+	imove (2 + (i % col_ht), (maxposslen + 8) * (i / col_ht));
 #else /* BOTTOMCONTEXT */
-	move (3 + contextsize + (i % col_ht), (maxposslen + 8) * (i / col_ht));
+	imove (3 + contextsize + (i % col_ht), (maxposslen + 8) * (i / col_ht));
 #endif /* BOTTOMCONTEXT */
 	if (i >= easypossibilities)
 	    (void) printf ("??: %s", possibilities[i]);
@@ -454,9 +478,9 @@ checkagain:
 	}
 
 #ifdef BOTTOMCONTEXT
-    move (li - contextsize - 1 - minimenusize, 0);
+    imove (li - contextsize - 1 - minimenusize, 0);
 #else /* BOTTOMCONTEXT */
-    move (2, 0);
+    imove (2, 0);
 #endif /* BOTTOMCONTEXT */
     for (i = contextsize;  --i > 0;  )
 	show_line (contextbufs[i], contextbufs[i], 0);
@@ -481,7 +505,7 @@ checkagain:
 
     if (minimenusize != 0)
 	{
-	move (li - 2, 0);
+	imove (li - 2, 0);
 	(void) printf (CORR_C_MINI_MENU);
 	}
 
@@ -492,10 +516,10 @@ checkagain:
 	    {
 	    case 'Z' & 037:
 		stop ();
-		erase ();
+		ierase ();
 		goto checkagain;
 	    case ' ':
-		erase ();
+		ierase ();
 		(void) fflush (stdout);
 		return;
 	    case 'q': case 'Q':
@@ -509,15 +533,16 @@ checkagain:
 		    c = 'y';
 		if (c == 'y' || c == 'Y')
 		    {
-		    erase ();
+		    ierase ();
 		    (void) fflush (stdout);
+		    fclose (outfile);	/* So `done' may unlink it safely */
 		    done (0);
 		    }
 		goto checkagain;
 	    case 'i': case 'I':
 		treeinsert (ichartosstr (strtosichar (ctok, 0), 1),
 		 ICHARTOSSTR_SIZE, 1);
-		erase ();
+		ierase ();
 		(void) fflush (stdout);
 		changes = 1;
 		return;
@@ -525,14 +550,14 @@ checkagain:
 		itok = strtosichar (ctok, 0);
 		lowcase (itok);
 		treeinsert (ichartosstr (itok, 1), ICHARTOSSTR_SIZE, 1);
-		erase ();
+		ierase ();
 		(void) fflush (stdout);
 		changes = 1;
 		return;
 	    case 'a': case 'A':
 		treeinsert (ichartosstr (strtosichar (ctok, 0), 1),
 		  ICHARTOSSTR_SIZE, 0);
-		erase ();
+		ierase ();
 		(void) fflush (stdout);
 		return;
 	    case 'L' & 037:
@@ -544,12 +569,12 @@ checkagain:
 		{
 		unsigned char	buf[200];
 
-		move (li - 1, 0);
+		imove (li - 1, 0);
 		(void) putchar ('!');
-		if (getline ((char *) buf) == NULL)
+		if (getline ((char *) buf, sizeof buf) == NULL)
 		    {
 		    (void) putchar (7);
-		    erase ();
+		    ierase ();
 		    (void) fflush (stdout);
 		    goto checkagain;
 		    }
@@ -560,18 +585,18 @@ checkagain:
 #else
 		(void) shellescape ((char *) buf);
 #endif
-		erase ();
+		ierase ();
 		goto checkagain;
 		}
 	    case 'r': case 'R':
-		move (li - 1, 0);
+		imove (li - 1, 0);
 		if (readonly)
 		    {
 		    (void) putchar (7);
 		    (void) printf ("%s ", CORR_C_READONLY);
 		    }
 		(void) printf (CORR_C_REPLACE_WITH);
-		if (getline ((char *) ctok) == NULL)
+		if (getline ((char *) ctok, ctokl) == NULL)
 		    {
 		    (void) putchar (7);
 		    /* Put it back */
@@ -591,7 +616,7 @@ checkagain:
 			}
 		    changes = 1;
 		    }
-		erase ();
+		ierase ();
 		if (icharlen (itok) <= minword)
 		    return;		/* Accept very short replacements */
 		goto checkagain;
@@ -618,10 +643,10 @@ checkagain:
 		    inserttoken (filteredbuf,
 		      filteredbuf + (begintoken - contextbufs[0]),
 		      ctok, curchar, 1);
-		    erase ();
+		    ierase ();
 		    if (readonly)
 			{
-			move (li - 1, 0);
+			imove (li - 1, 0);
 			(void) putchar (7);
 			(void) printf ("%s", CORR_C_READONLY);
 			(void) fflush (stdout);
@@ -637,23 +662,23 @@ checkagain:
 	    case 'l': case 'L':
 		{
 		unsigned char	buf[100];
-		move (li - 1, 0);
+		imove (li - 1, 0);
 		(void) printf (CORR_C_LOOKUP_PROMPT);
-		if (getline ((char *) buf) == NULL)
+		if (getline ((char *) buf, sizeof buf) == NULL)
 		    {
 		    (void) putchar (7);
-		    erase ();
+		    ierase ();
 		    goto checkagain;
 		    }
 		(void) printf ("\r\n");
 		(void) fflush (stdout);
 		lookharder (buf);
-		erase ();
+		ierase ();
 		goto checkagain;
 		}
 	    case 'x': case 'X':
 		quit = 1;
-		erase ();
+		ierase ();
 		(void) fflush (stdout);
 		return;
 	    default:
@@ -794,11 +819,20 @@ static void inserttoken (buf, start, tok, curchar, oktochange)
 			q;
     unsigned char *	ew;
 
+    if (!oktochange  &&  logfile != NULL)
+	{
+	for (p = start;  p != *curchar;  p++)
+	    (void) putc (*p, logfile);
+	(void) putc (' ', logfile);
+	(void) fputs (tok, logfile);
+	(void) putc ('\n', logfile);
+	(void) fflush (logfile);
+	}
+
     (void) strcpy ((char *) copy, (char *) buf);
 
-    for (p = buf, q = copy; p != start; p++, q++)
-	*p = *q;
-    q += *curchar - start;
+    p = start;
+    q = copy + (*curchar - buf);
     ew = skipoverword (tok);
     while (tok < ew)
 	*p++ = *tok++;
@@ -926,6 +960,78 @@ void makepossibilities (word)
 	}
     }
 
+int inpossibilities (ctok)
+    register unsigned char *	ctok;
+    {
+    register int		i;
+
+    /*
+     * This function is a horrible kludge, necessitated by a problem
+     * that shows up in German (and possibly other languages).  The
+     * problem in German is that the character "ess-zed" (which I'm
+     * going to write as "|3" in these comments so I don't have to use
+     * the ISO Latin-1 character set) doesn't have an uppercase
+     * equivalent.  Instead, words that contain |3 are uppercased by
+     * converting the character to SS.
+     *
+     * For ispell, this presents a problem.  Consider the two German
+     * words "gro|3" (large) and "nass" (wet).  In lowercase, they are
+     * represented differently both externally and internally.
+     * Internally, the "ss" gets converted to a single ichar_t (for a
+     * couple of good reasons I won't go into right now).  Internally,
+     * there is also an uppercase representation for |3.  So when we
+     * do things like dictionary lookups (which are in uppercase for
+     * historical reasons) we can distinguish the correct "gro|3" and
+     * "nass" from the incorrect "gross" and "na|3", even when they're
+     * converted to uppercase.
+     *
+     * The problem arises when the user writes the words in uppercase:
+     * "GROSS" and "NASS".  When parsing the character strings, ispell
+     * would like to convert the "SS" sequence into the correct
+     * ichar_t for the word.  However, the ichar_t that is needed for
+     * the two S's in "GROSS" (uppercase version of |3) is different
+     * from the ichar_t needed for the two S's in "NASS" (SS).  There
+     * is no solution to the problem that can be based purely on
+     * lexical information; the character representation simply
+     * doesn't have the information needed.
+     *
+     * What ispell really needs is to base the chose of ichar_t
+     * representation on the correct spelling of the word.  For
+     * "GROSS" it should pick uppercase |3 because "gro|3" is in the
+     * dictionary, and for "NASS" it should pick SS for the same
+     * reason.
+     *
+     * That brings us to this function.  Lexically, ispell will always
+     * choose one or the other of the internal representations.  (The
+     * choice is fairly unpredictable; it depends on which one the
+     * binary search happens to hit upon first.)  If good() says that
+     * the spelling is OK, we must have chosen the right
+     * representation (and we'll never get into this function.  But if
+     * good() fails, makepossibilities() will wind up substituting the
+     * correct character for the incorrect one in wrongletter().  Then
+     * the reconversion to external string format will wind up
+     * generating exactly the word that was originally spell-checked.
+     *
+     * So this function searches the possibilities list to see if the
+     * requested word is found in the list.  If so, the caller will
+     * accept the word just as if good() had succeeded.
+     *
+     * This is a pretty ugly solution.  It's also only partial.  If a
+     * word contains TWO ambiguous characters, it won't find a
+     * solution because wrongletter() can only correct a single error.
+     * Fortunately, at least in German, that's not a huge problem.
+     * The most popular German dictionary contains only three words
+     * that have more than one ess-zed: Ku|3tengewa|3ser,
+     * Vergro|3Serungsgla|3er, and gro|3Senordnungsma|3Sig/A.
+     */
+    for (i = 0;  i < pcount;  i++)
+	{
+	if (strcmp ((char *) ctok, possibilities[i]) == 0)
+	    return 1;
+	}
+    return 0;
+    }
+
 static int insert (word)
     register ichar_t *	word;
     {
@@ -1477,8 +1583,9 @@ static void save_root_cap (word, pattern, prestrip, preadd, sufstrip, sufadd,
     return;
     }
 
-static char * getline (s)
+static char * getline (s, len)
     register char *	s;
+    register int	len;
     {
     register char *	p;
     register int	c;
@@ -1489,6 +1596,14 @@ static char * getline (s)
 	{
 	(void) fflush (stdout);
 	c = GETKEYSTROKE ();
+	/*
+	** Don't let them overflow the buffer.
+	*/
+	if (p >= s + len - 1)
+	    {
+	    *p = 0;
+	    return s;
+	    }
 	if (c == '\\')
 	    {
 	    (void) putchar ('\\');
@@ -1548,7 +1663,8 @@ void askmode ()
 	{
 	if (freopen (askfilename, "w", stdout) == NULL)
 	    {
-	    (void) fprintf (stderr, CANT_CREATE, askfilename);
+	    (void) fprintf (stderr, CANT_CREATE, askfilename,
+	      MAYBE_CR (stderr));
 	    exit (1);
 	    }
 	}
@@ -1600,7 +1716,9 @@ void askmode ()
 		ch = getc (stdin);
 		if (ch == EOF)
 		    break;
+		contextbufs[0][bufsize] = (char) ch;
 		filteredbuf[bufsize++] = (char) ch;
+		contextbufs[0][bufsize] = '\0';
 		filteredbuf[bufsize] = '\0';
 		}
 	    }
@@ -1672,10 +1790,10 @@ void askmode ()
 		{
 		if (hadnl)
 		    filteredbuf[bufsize - 1] = '\0';
-		defdupchar =
+		defstringgroup =
 		  findfiletype ((char *) &filteredbuf[1], 1, (int *) NULL);
-		if (defdupchar < 0)
-		    defdupchar = 0;
+		if (defstringgroup < 0)
+		    defstringgroup = 0;
 		if (hadnl)
 		    filteredbuf[bufsize - 1] = '\n';
 		}
@@ -1728,7 +1846,7 @@ void copyout (cc, cnt)
     {
     register char *	cp;		/* Char in contextbufs[0] to copy */
     
-    cp = &contextbufs[0][*cc - filteredbuf];
+    cp = (char *) &contextbufs[0][*cc - filteredbuf];
     *cc += cnt;
     while (--cnt >= 0)
 	{
