@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: defmt.c,v 1.58 2001/08/01 22:15:56 geoff Exp $";
+    "$Id: defmt.c,v 1.62 2005/04/20 23:16:32 geoff Exp $";
 #endif
 
 /*
@@ -27,16 +27,8 @@ static char Rcs_Id[] =
  *    such.  Binary redistributions based on modified source code
  *    must be clearly marked as modified versions in the documentation
  *    and/or other materials provided with the distribution.
- * 4. Any web site or other electronic service that offers ispell for
- *    download or other electronic transfer as a separate entity, in
- *    either source or binary form, must also include a prominent statement
- *    indicating that information about ispell can be obtained from the
- *    following Web site URL:
- *	http://fmg-www.cs.ucla.edu/geoff/ispell.html
- *    If the offering service supports hyperlinks, the aforementioned
- *    Web site must also be offered as a hyperlink.  Condition #4 does
- *    not apply if ispell is offered only as part of a larger, aggregated
- *    product such as a word processor or packaged operating system.
+ * 4. The code that causes the 'ispell -v' command to display a prominent
+ *    link to the official ispell Web site may not be removed.
  * 5. The name of Geoff Kuenning may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
@@ -60,6 +52,19 @@ static char Rcs_Id[] =
 
 /*
  * $Log: defmt.c,v $
+ * Revision 1.62  2005/04/20 23:16:32  geoff
+ * Use inpossibilities to deal with the case where uppercase SS in German
+ * causes ambiguities.
+ *
+ * Revision 1.61  2005/04/14 14:38:23  geoff
+ * Update license.
+ *
+ * Revision 1.60  2001/11/28 22:39:26  geoff
+ * Add Ken Stevens's fix for newlines in \verb
+ *
+ * Revision 1.59  2001/09/06 00:30:29  geoff
+ * Many changes from Eli Zaretskii to support DJGPP compilation.
+ *
  * Revision 1.58  2001/08/01 22:15:56  geoff
  * When processing quoted strings inside HTML tags, don't handle
  * ampersand sequences unless the quoted string is being spell-checked.
@@ -255,7 +260,7 @@ static unsigned char * skiptoword (bufp) /* Skip to beginning of a word */
     while (*bufp
       &&  ((!isstringch (bufp, 0)  &&  !iswordch (chartoichar (*bufp)))
 	||  isboundarych (chartoichar (*bufp))
-	||  (tflag == DEFORMAT_TEX  &&  (math_mode & 1))
+	||  (tflag == DEFORMAT_TEX  &&  ((math_mode & 1) || LaTeX_Mode != 'P'))
 	||  (insidehtml & (HTML_IN_SPEC | HTML_ISIGNORED)) != 0
 	||  ((insidehtml & HTML_IN_TAG) != 0
 	  &&  (insidehtml
@@ -390,7 +395,7 @@ static unsigned char * skiptoword (bufp) /* Skip to beginning of a word */
 	    /* Odd numbers mean we are in "math mode" */
 	    /* Even numbers mean we are in LR or */
 	    /* paragraph mode */
-	    if (*bufp == TEXPERCENT)
+	    if (*bufp == TEXPERCENT  &&  LaTeX_Mode != 'v')
 		{
 		if (!TeX_comment)
 		    {
@@ -417,7 +422,8 @@ static unsigned char * skiptoword (bufp) /* Skip to beginning of a word */
 		    }
 		if (math_mode < 0)
 		    {
-		    (void) fprintf (stderr, DEFMT_C_TEX_MATH_ERROR);
+		    (void) fprintf (stderr, DEFMT_C_TEX_MATH_ERROR,
+		      MAYBE_CR (stderr));
 		    math_mode = 0;
 		    }
 		}
@@ -446,6 +452,14 @@ static unsigned char * skiptoword (bufp) /* Skip to beginning of a word */
 		    TeX_skip_parens(&bufp);
 		    LaTeX_Mode = 'P';
 		    }
+		else if (LaTeX_Mode == 'v')
+		    {
+		    /* continued "verb" */
+		    while (*bufp != save_LaTeX_Mode  &&  *bufp != '\0')
+			bufp++;
+		    if (*bufp != 0)
+			LaTeX_Mode = 'P';
+		    }
 		else if (TeX_math_begin(&bufp))
 		    /* checks references and */
 		    /* skips \ commands */
@@ -763,7 +777,23 @@ void checkline (ofile)
 		else
 		    {
 		    makepossibilities (itoken);
-		    if (pcount)
+		    if (inpossibilities (ctoken)) /* Kludge for German, etc. */
+			{
+			/* might not be perfect match, but we'll lie */
+			if (!terse)
+			    {
+			    if (askverbose)
+				(void) fprintf (ofile, "ok\n");
+			    else
+				{
+				if (correct_verbose_mode)
+				    (void) fprintf (ofile, "* %s\n", ctoken );
+				else
+				    (void) fprintf (ofile, "*\n");
+				}
+			    }
+			}
+		    else if (pcount)
 			{
 			/*
 			** print &  or ?, ctoken, then
@@ -983,7 +1013,8 @@ static int TeX_LR_check (begin_p, bufp)
 	    math_mode -= (math_mode & 127) * 128;
 	    if (math_mode < 0)
 		{
-		(void) fprintf (stderr, DEFMT_C_LR_MATH_ERROR);
+		(void) fprintf (stderr, DEFMT_C_LR_MATH_ERROR,
+		  MAYBE_CR (stderr));
 		math_mode = 1;
 		}
 	    }
@@ -1112,6 +1143,12 @@ static void TeX_skip_check (bufp)
 	*bufp += 5;
 	while (**bufp != skip_ch  &&  **bufp != '\0')
 	    (*bufp)++;
+	/* skip to end of verb field when not in a comment or math field */
+	if (**bufp == 0 && !TeX_comment  &&  !(math_mode & 1))
+	    {
+	    LaTeX_Mode = 'v';
+	    save_LaTeX_Mode = skip_ch;
+	    }
 	}
     else
 	{
@@ -1146,7 +1183,7 @@ static void TeX_skip_check (bufp)
  * header, but I doubt that it varies, and I don't want to change the
  * syntax of affix files right now.
  *
- * Incidentally, TeX_strncmp uses uneual signedness for its arguments
+ * Incidentally, TeX_strncmp uses unequal signedness for its arguments
  * because that's how it's always called, and it's easier to do one
  * typecast here than lots of casts in the calls.
  */
@@ -1236,7 +1273,7 @@ int init_keyword_table (rawtags, envvar, deftags, ignorecase, keywords)
     wlist = malloc (wsize);
     if (wlist == NULL)
 	{
-	(void) fprintf (stderr, DEFMT_C_NO_SPACE);
+	(void) fprintf (stderr, DEFMT_C_NO_SPACE, MAYBE_CR (stderr));
 	exit (1);
 	}
     wlist[0] = '\0';
@@ -1275,7 +1312,7 @@ int init_keyword_table (rawtags, envvar, deftags, ignorecase, keywords)
       (unsigned char **) malloc (keywords->numkw * sizeof keywords->kwlist[0]);
     if (keywords->kwlist == NULL)
 	{
-	fprintf (stderr, DEFMT_C_NO_SPACE);
+	fprintf (stderr, DEFMT_C_NO_SPACE, MAYBE_CR (stderr));
 	exit (1);
 	}
 
@@ -1318,7 +1355,7 @@ int init_keyword_table (rawtags, envvar, deftags, ignorecase, keywords)
 	  malloc ((maxkeywordlen + 1) * sizeof keywordbuf[0]);
 	if (keywordbuf == NULL)
 	    {
-	    fprintf (stderr, DEFMT_C_NO_SPACE);
+	    fprintf (stderr, DEFMT_C_NO_SPACE, MAYBE_CR (stderr));
 	    exit(1);
 	    }
 	}
