@@ -1,5 +1,7 @@
+#ifndef MSGS_H_INCLUDED
+#define MSGS_H_INCLUDED
 /*
- * $Id: msgs.h,v 1.40 2001/07/25 21:51:47 geoff Exp $
+ * $Id: msgs.h,v 1.45 2005/04/28 00:19:48 geoff Exp $
  *
  * Copyright 1992, 1993, 1999, 2001, Geoff Kuenning, Claremont, CA
  * All rights reserved.
@@ -17,16 +19,8 @@
  *    such.  Binary redistributions based on modified source code
  *    must be clearly marked as modified versions in the documentation
  *    and/or other materials provided with the distribution.
- * 4. Any web site or other electronic service that offers ispell for
- *    download or other electronic transfer as a separate entity, in
- *    either source or binary form, must also include a prominent statement
- *    indicating that information about ispell can be obtained from the
- *    following Web site URL:
- *	http://fmg-www.cs.ucla.edu/geoff/ispell.html
- *    If the offering service supports hyperlinks, the aforementioned
- *    Web site must also be offered as a hyperlink.  Condition #4 does
- *    not apply if ispell is offered only as part of a larger, aggregated
- *    product such as a word processor or packaged operating system.
+ * 4. The code that causes the 'ispell -v' command to display a prominent
+ *    link to the official ispell Web site may not be removed.
  * 5. The name of Geoff Kuenning may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
@@ -71,6 +65,21 @@
 
 /*
  * $Log: msgs.h,v $
+ * Revision 1.45  2005/04/28 00:19:48  geoff
+ * Remove obsolete messages related to the count file.
+ *
+ * Revision 1.44  2005/04/26 23:57:22  geoff
+ * Add double-inclusion protection
+ *
+ * Revision 1.43  2005/04/20 23:06:32  geoff
+ * Add a new message and tweak an old one.
+ *
+ * Revision 1.42  2005/04/13 22:52:37  geoff
+ * Update the license.  Improve a couple of messages.
+ *
+ * Revision 1.41  2001/09/06 00:34:38  geoff
+ * Many changes from Eli Zaretskii to support DJGPP compilation.
+ *
  * Revision 1.40  2001/07/25 21:51:47  geoff
  * *** empty log message ***
  *
@@ -127,26 +136,32 @@
  *
  */
 
+/*
+ * Produce either a CR or an empty string, depending on whether
+ * stderr is a terminal device or not.  Used when the message
+ * could be printed to a redirected stream, to avoid the pesky ^M.
+ */
+#define MAYBE_CR(stream) (isatty (fileno (stream)) ? "\r" : "")
+
 /*
  * The following strings are used in numerous places:
  */
-#define BAD_FLAG	"\r\nIllegal affix flag character '%c'\r\n"
-#define CANT_OPEN	"Can't open %s\r\n"
-#define CANT_CREATE	"Can't create %s\r\n"
-#define WORD_TOO_LONG(w) "\r\nWord '%s' too long at line %d of %s, truncated\r\n", \
-			  w, __LINE__, __FILE__
+#define BAD_FLAG	"%s\nIllegal affix flag character '%c'%s\n"
+#define CANT_OPEN	"Can't open %s%s\n"
+#define CANT_CREATE	"Can't create %s%s\n"
+#define WORD_TOO_LONG(w) "%s\nWord '%s' too long at line %d of %s, truncated%s\n", \
+			  MAYBE_CR (stderr), w, __LINE__, __FILE__, \
+			  MAYBE_CR (stderr)
 
 /*
  * The following strings are used in buildhash.c:
  */
 #define BHASH_C_NO_DICT		"No dictionary (%s)\n"
-#define BHASH_C_NO_COUNT	"No count file\n"
-#define BHASH_C_BAD_COUNT	"Bad count file\n"
 #define BHASH_C_ZERO_COUNT	"No words in dictionary\n"
     /* I think this message looks better when it's nearly 80 characters wide,
      * thus the ugly formatting in the next two defines.  GK 9-87 */
 #define BHASH_C_BAFF_1(max, excess) \
-  "    Warning:  this language table may exceed the maximum total affix length\nof %d by up to %d bytes.  You should either increase MAXAFFIXLEN in config.X\nor shorten your largest affix/strip string difference.  (This is the\n", \
+  "    Warning:  this language table may exceed the maximum total affix length\nof %d by up to %d bytes.  You should either increase MAXAFFIXLEN in local.h\nor shorten your largest affix/strip string difference.  (This is the\n", \
 				  max, excess
 #define BHASH_C_BAFF_2 \
   "difference between the affix length and the strip length in a given\nreplacement rule, or the affix length if there is no strip string\nin that rule.)\n"
@@ -161,30 +176,30 @@
 /*
  * The following strings are used in correct.c:
  */
-#define CORR_C_HELP_1		"Whenever a word is found that is not in the dictionary,\r\n"
-#define CORR_C_HELP_2		"it is printed on the first line of the screen.  If the dictionary\r\n"
-#define CORR_C_HELP_3		"contains any similar words, they are listed with a number\r\n"
-#define CORR_C_HELP_4		"next to each one.  You have the option of replacing the word\r\n"
-#define CORR_C_HELP_5		"completely, or choosing one of the suggested words.\r\n"
+#define CORR_C_HELP_1		"Whenever a word is found that is not in the dictionary,%s\n"
+#define CORR_C_HELP_2		"it is printed on the first line of the screen.  If the dictionary%s\n"
+#define CORR_C_HELP_3		"contains any similar words, they are listed with a number%s\n"
+#define CORR_C_HELP_4		"next to each one.  You have the option of replacing the word%s\n"
+#define CORR_C_HELP_5		"completely, or choosing one of the suggested words.%s\n"
     /* You may add HELP_6 through HELP_9 if your language needs more lines */
 #define CORR_C_HELP_6		""
 #define CORR_C_HELP_7		""
 #define CORR_C_HELP_8		""
 #define CORR_C_HELP_9		""
-#define CORR_C_HELP_COMMANDS	"\r\nCommands are:\r\n\r\n"
-#define CORR_C_HELP_R_CMD	"R       Replace the misspelled word completely.\r\n"
-#define CORR_C_HELP_BLANK	"Space   Accept the word this time only.\r\n"
-#define CORR_C_HELP_A_CMD	"A       Accept the word for the rest of this session.\r\n"
-#define CORR_C_HELP_I_CMD	"I       Accept the word, and put it in your private dictionary.\r\n"
-#define CORR_C_HELP_U_CMD	"U       Accept and add lowercase version to private dictionary.\r\n"
-#define CORR_C_HELP_0_CMD	"0-n     Replace with one of the suggested words.\r\n"
-#define CORR_C_HELP_L_CMD	"L       Look up words in system dictionary.\r\n"
-#define CORR_C_HELP_X_CMD	"X       Write the rest of this file, ignoring misspellings,\r\n        and start next file.\r\n"
-#define CORR_C_HELP_Q_CMD	"Q       Quit immediately.  Asks for confirmation.\r\n        Leaves file unchanged.\r\n"
-#define CORR_C_HELP_BANG	"!       Shell escape.\r\n"
-#define CORR_C_HELP_REDRAW	"^L      Redraw screen.\r\n"
-#define CORR_C_HELP_SUSPEND	"^Z      Suspend program.\r\n"
-#define CORR_C_HELP_HELP	"?       Show this help screen.\r\n"
+#define CORR_C_HELP_COMMANDS	"%s\nCommands are:%s\n%s\n"
+#define CORR_C_HELP_R_CMD	"R       Replace the misspelled word completely.%s\n"
+#define CORR_C_HELP_BLANK	"Space   Accept the word this time only.%s\n"
+#define CORR_C_HELP_A_CMD	"A       Accept the word for the rest of this session.%s\n"
+#define CORR_C_HELP_I_CMD	"I       Accept the word, and put it in your private dictionary.%s\n"
+#define CORR_C_HELP_U_CMD	"U       Accept and add lowercase version to private dictionary.%s\n"
+#define CORR_C_HELP_0_CMD	"0-n     Replace with one of the suggested words.%s\n"
+#define CORR_C_HELP_L_CMD	"L       Look up words in system dictionary.%s\n"
+#define CORR_C_HELP_X_CMD	"X       Write the rest of this file, ignoring misspellings,%s\n        and start next file.%s\n"
+#define CORR_C_HELP_Q_CMD	"Q       Quit immediately.  Asks for confirmation.%s\n        Leaves file unchanged.%s\n"
+#define CORR_C_HELP_BANG	"!       Shell escape.%s\n"
+#define CORR_C_HELP_REDRAW	"^L      Redraw screen.%s\n"
+#define CORR_C_HELP_SUSPEND	"^Z      Suspend program.%s\n"
+#define CORR_C_HELP_HELP	"?       Show this help screen.%s\n"
 #define CORR_C_HELP_TYPE_SPACE	"-- Type space to continue --"
 
 #define CORR_C_FILE_LABEL	"              File: %s"
@@ -196,14 +211,14 @@
 #define CORR_C_MORE_PROMPT	"-- more --"
 #define CORR_C_BLANK_MORE	"\r           \r"
 #define CORR_C_END_LOOK		"--end--"
-#define CORR_C_SHORT_SOURCE	"ispell:  unexpected EOF on unfiltered version of input\r\n"
+#define CORR_C_SHORT_SOURCE	"ispell:  unexpected EOF on unfiltered version of input%s\n"
 
 /*
  * The following strings are used in defmt.c:
  */
-#define DEFMT_C_TEX_MATH_ERROR	"***ERROR in parsing TeX math mode!\r\n"
-#define DEFMT_C_LR_MATH_ERROR	"***ERROR in LR to math-mode switch.\r\n"
-#define DEFMT_C_NO_SPACE	"Ran out of space building keyword list\r\n"
+#define DEFMT_C_TEX_MATH_ERROR	"***ERROR in parsing TeX math mode!%s\n"
+#define DEFMT_C_LR_MATH_ERROR	"***ERROR in LR to math-mode switch.%s\n"
+#define DEFMT_C_NO_SPACE	"Ran out of space building keyword list%s\n"
 
 /*
  * The following strings are used in icombine.c:
@@ -225,41 +240,42 @@
 #define ISPELL_C_USAGE5		"       %s [-dfile] [-wchars] -e[1-4]\n"
 #define ISPELL_C_USAGE6		"       %s [-dfile] [-wchars] -D\n"
 #define ISPELL_C_USAGE7		"       %s -v\n"
-#define ISPELL_C_TEMP_DISAPPEARED "temporary file disappeared (%s)\r\n"
+#define ISPELL_C_TEMP_DISAPPEARED "temporary file disappeared or is unreadable (%s)%s\n"
 #define ISPELL_C_BAD_TYPE	"ispell:  unrecognized formatter type '%s'\n"
 #define ISPELL_C_NO_FILE	"ispell:  specified file does not exist\n"
 #define ISPELL_C_NO_FILES	"ispell:  specified files do not exist\n"
-#define ISPELL_C_CANT_WRITE	"Warning:  Can't write to %s\r\n"
+#define ISPELL_C_CANT_WRITE	"Warning:  Can't write to %s%s\n"
 #define ISPELL_C_OPTIONS_ARE	"Compiled-in options:\n"
-#define ISPELL_C_UNEXPECTED_FD	"ispell:  unexpected fd while opening '%s'\r\n"
+#define ISPELL_C_UNEXPECTED_FD	"ispell:  unexpected fd while opening '%s'%s\n"
+#define ISPELL_C_NO_OPTIONS_SPACE "ispell: no memory to read default options\n"
 
 /*
  * The following strings are used in lookup.c:
  */
-#define LOOKUP_C_CANT_READ	"Trouble reading hash table %s\r\n"
-#define LOOKUP_C_NULL_HASH	"Null hash table %s\r\n"
+#define LOOKUP_C_CANT_READ	"Trouble reading hash table %s%s\n"
+#define LOOKUP_C_NULL_HASH	"Null hash table %s%s\n"
 #define LOOKUP_C_SHORT_HASH(name, gotten, wanted) \
-				"Truncated hash table %s:  got %d bytes, expected %d\r\n", \
+				"Truncated hash table %s:  got %d bytes, expected %d%s\n", \
 				  name, gotten, wanted
 #define LOOKUP_C_BAD_MAGIC(name, wanted, gotten) \
-				"Illegal format hash table %s - expected magic 0x%x, got 0x%x\r\n", \
+				"Illegal format hash table %s - expected magic 0x%x, got 0x%x%s\n", \
 				  name, wanted, gotten
 #define LOOKUP_C_BAD_MAGIC2(name, wanted, gotten) \
-				"Illegal format hash table %s - expected magic2 0x%x, got 0x%x\r\n", \
+				"Illegal format hash table %s - expected magic2 0x%x, got 0x%x%s\n", \
 				  name, wanted, gotten
 #define LOOKUP_C_BAD_OPTIONS(gotopts, gotchars, gotlen, wantedopts, wantedchars, wantedlen) \
-				"Hash table options don't agree with buildhash - 0x%x/%d/%d vs. 0x%x/%d/%d\r\n", \
+				"Hash table options don't agree with buildhash - 0x%x/%d/%d vs. 0x%x/%d/%d%s\n", \
 				  gotopts, gotchars, gotlen, \
 				  wantedopts, wantedchars, wantedlen
-#define LOOKUP_C_NO_HASH_SPACE	"Couldn't allocate space for hash table\r\n"
-#define LOOKUP_C_BAD_FORMAT	"Illegal format hash table\r\n"
-#define LOOKUP_C_NO_LANG_SPACE	"Couldn't allocate space for language tables\r\n"
+#define LOOKUP_C_NO_HASH_SPACE	"Couldn't allocate space for hash table%s\n"
+#define LOOKUP_C_BAD_FORMAT	"Illegal format hash table%s\n"
+#define LOOKUP_C_NO_LANG_SPACE	"Couldn't allocate space for language tables%s\n"
 
 /*
  * The following strings are used in makedent.c:
  */
-#define MAKEDENT_C_NO_WORD_SPACE "\r\nCouldn't allocate space for word '%s'\r\n"
-#define MAKEDENT_C_BAD_WORD_CHAR "\r\nWord '%s' contains illegal characters\r\n"
+#define MAKEDENT_C_NO_WORD_SPACE "%s\nCouldn't allocate space for word '%s'%s\n"
+#define MAKEDENT_C_BAD_WORD_CHAR "%s\nWord '%s' contains illegal characters%s\n"
 
 /*
  * The following strings are used in parse.y:
@@ -271,7 +287,9 @@
 #define PARSE_Y_NULL_STRING	"String character must have nonzero length"
 #define PARSE_Y_MANY_STRINGS	"Too many string characters"
 #define PARSE_Y_NO_SUCH_STRING	"No such string character"
-#define PARSE_Y_MULTIPLE_STRINGS "Alternate string character was already defined"
+#define PARSE_Y_MULTIPLE_STRINGS "Alternate string character is already defined"
+#define PARSE_Y_WRONG_STRING_COUNT \
+				"Alternate string characters must map one-to-one to base string characters"
 #define PARSE_Y_LENGTH_MISMATCH	"Upper and lower versions of string character must be same length"
 #define PARSE_Y_WRONG_NROFF	"Incorrect character count in nroffchars statement"
 #define PARSE_Y_WRONG_TEX	"Incorrect character count in TeXchars statement"
@@ -296,7 +314,7 @@
  */
 #define TERM_C_SMALL_SCREEN	"Screen too small:  need at least %d lines\n"
 #define TERM_C_NO_BATCH		"Can't deal with non-interactive use yet.\n"
-#define TERM_C_CANT_FORK	"Couldn't fork, try later.\r\n"
+#define TERM_C_CANT_FORK	"Couldn't fork, try later.%s\n"
 #define TERM_C_TYPE_SPACE	"\n-- Type space to continue --"
 
 /*
@@ -307,12 +325,14 @@
 /*
  * The following strings are used in tree.c:
  */
-#define TREE_C_CANT_UPDATE	"Warning: Cannot update personal dictionary (%s)\r\n"
-#define TREE_C_NO_SPACE		"Ran out of space for personal dictionary\r\n"
-#define TREE_C_TRY_ANYWAY	"Continuing anyway (with reduced performance).\r\n"
+#define TREE_C_CANT_UPDATE	"Warning: Cannot update personal dictionary (%s)%s\n"
+#define TREE_C_NO_SPACE		"Ran out of space for personal dictionary%s\n"
+#define TREE_C_TRY_ANYWAY	"Continuing anyway (with reduced performance).%s\n"
 
 /*
  * The following strings are used in unsq.c:
  */
 #define UNSQ_C_BAD_COUNT	"Illegal count character 0x%x\n"
 #define UNSQ_C_SURPRISE_EOF	"Unexpected EOF\n"
+
+#endif /* MSGS_H_INCLUDED */
