@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: lookup.c,v 1.49 2001/07/25 21:51:46 geoff Exp $";
+    "$Id: lookup.c,v 1.52 2005/04/14 21:25:52 geoff Exp $";
 #endif
 
 /*
@@ -8,7 +8,7 @@ static char Rcs_Id[] =
  *
  * Pace Willisson, 1983
  *
- * Copyright 1987, 1988, 1989, 1992, 1993, 1999, 2001, Geoff Kuenning,
+ * Copyright 1987, 1988, 1989, 1992, 1993, 1999, 2001, 2005, Geoff Kuenning,
  * Claremont, CA.
  * All rights reserved.
  *
@@ -25,16 +25,8 @@ static char Rcs_Id[] =
  *    such.  Binary redistributions based on modified source code
  *    must be clearly marked as modified versions in the documentation
  *    and/or other materials provided with the distribution.
- * 4. Any web site or other electronic service that offers ispell for
- *    download or other electronic transfer as a separate entity, in
- *    either source or binary form, must also include a prominent statement
- *    indicating that information about ispell can be obtained from the
- *    following Web site URL:
- *	http://fmg-www.cs.ucla.edu/geoff/ispell.html
- *    If the offering service supports hyperlinks, the aforementioned
- *    Web site must also be offered as a hyperlink.  Condition #4 does
- *    not apply if ispell is offered only as part of a larger, aggregated
- *    product such as a word processor or packaged operating system.
+ * 4. The code that causes the 'ispell -v' command to display a prominent
+ *    link to the official ispell Web site may not be removed.
  * 5. The name of Geoff Kuenning may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
@@ -54,6 +46,15 @@ static char Rcs_Id[] =
 
 /*
  * $Log: lookup.c,v $
+ * Revision 1.52  2005/04/14 21:25:52  geoff
+ * Fix a couple of tiny formatting inconsistencies.
+ *
+ * Revision 1.51  2005/04/14 14:38:23  geoff
+ * Update license.  Improve some typecasting.
+ *
+ * Revision 1.50  2001/09/06 00:30:28  geoff
+ * Many changes from Eli Zaretskii to support DJGPP compilation.
+ *
  * Revision 1.49  2001/07/25 21:51:46  geoff
  * Minor license update.
  *
@@ -113,7 +114,7 @@ int linit ()
 
     if ((hashfd = open (hashname, 0 | MSDOS_BINARY_OPEN)) < 0)
 	{
-	(void) fprintf (stderr, CANT_OPEN, hashname);
+	(void) fprintf (stderr, CANT_OPEN, hashname, MAYBE_CR (stderr));
 	return (-1);
 	}
 
@@ -121,27 +122,29 @@ int linit ()
     if (hashsize < sizeof hashheader)
 	{
 	if (hashsize == (unsigned int) -1)
-	    (void) fprintf (stderr, LOOKUP_C_CANT_READ, hashname);
+	    (void) fprintf (stderr, LOOKUP_C_CANT_READ, hashname,
+	      MAYBE_CR (stderr));
 	else if (hashsize == 0)
-	    (void) fprintf (stderr, LOOKUP_C_NULL_HASH, hashname);
+	    (void) fprintf (stderr, LOOKUP_C_NULL_HASH, hashname,
+	      MAYBE_CR (stderr));
 	else
 	    (void) fprintf (stderr,
 	      LOOKUP_C_SHORT_HASH (hashname, hashsize,
-	        (int) sizeof hashheader));
+	        (int) sizeof hashheader), MAYBE_CR (stderr));
 	return (-1);
 	}
     else if (hashheader.magic != MAGIC)
 	{
 	(void) fprintf (stderr,
 	  LOOKUP_C_BAD_MAGIC (hashname, (unsigned int) MAGIC,
-	    (unsigned int) hashheader.magic));
+	    (unsigned int) hashheader.magic), MAYBE_CR (stderr));
 	return (-1);
 	}
     else if (hashheader.magic2 != MAGIC)
 	{
 	(void) fprintf (stderr,
 	  LOOKUP_C_BAD_MAGIC2 (hashname, (unsigned int) MAGIC,
-	    (unsigned int) hashheader.magic2));
+	    (unsigned int) hashheader.magic2), MAYBE_CR (stderr));
 	return (-1);
 	}
     else if (hashheader.compileoptions != COMPILEOPTIONS
@@ -151,7 +154,8 @@ int linit ()
 	(void) fprintf (stderr,
 	  LOOKUP_C_BAD_OPTIONS ((unsigned int) hashheader.compileoptions,
 	    hashheader.maxstringchars, hashheader.maxstringcharlen,
-	    (unsigned int) COMPILEOPTIONS, MAXSTRINGCHARS, MAXSTRINGCHARLEN));
+	    (unsigned int) COMPILEOPTIONS, MAXSTRINGCHARS, MAXSTRINGCHARLEN),
+	    MAYBE_CR (stderr));
 	return (-1);
 	}
     if (nodictflag)
@@ -167,7 +171,7 @@ int linit ()
 	hashtbl = (struct dent *) calloc (1, sizeof (struct dent));
 	if (hashtbl == NULL)
 	    {
-	    (void) fprintf (stderr, LOOKUP_C_NO_HASH_SPACE);
+	    (void) fprintf (stderr, LOOKUP_C_NO_HASH_SPACE, MAYBE_CR (stderr));
 	    return (-1);
 	    }
 	hashtbl[0].word = NULL;
@@ -192,7 +196,7 @@ int linit ()
       malloc ((numsflags + numpflags) * sizeof (struct flagent));
     if (hashtbl == NULL  ||  hashstrings == NULL  ||  sflaglist == NULL)
 	{
-	(void) fprintf (stderr, LOOKUP_C_NO_HASH_SPACE);
+	(void) fprintf (stderr, LOOKUP_C_NO_HASH_SPACE, MAYBE_CR (stderr));
 	return (-1);
 	}
     pflaglist = sflaglist + numsflags;
@@ -207,7 +211,7 @@ int linit ()
 	if (read (hashfd, hashstrings, (unsigned) hashheader.lstringsize)
 	  != (int) hashheader.lstringsize)
 	    {
-	    (void) fprintf (stderr, LOOKUP_C_BAD_FORMAT);
+	    (void) fprintf (stderr, LOOKUP_C_BAD_FORMAT, MAYBE_CR (stderr));
 	    return (-1);
 	    }
 	(void) lseek (hashfd,
@@ -223,7 +227,7 @@ int linit ()
 	      (unsigned) hashheader.tblsize * sizeof (struct dent))
 	    != (int) (hashheader.tblsize * sizeof (struct dent)))
 	    {
-	    (void) fprintf (stderr, LOOKUP_C_BAD_FORMAT);
+	    (void) fprintf (stderr, LOOKUP_C_BAD_FORMAT, MAYBE_CR (stderr));
 	    return (-1);
 	    }
 	}
@@ -231,7 +235,7 @@ int linit ()
 	(unsigned) (numsflags + numpflags) * sizeof (struct flagent))
       != (numsflags + numpflags) * sizeof (struct flagent))
 	{
-	(void) fprintf (stderr, LOOKUP_C_BAD_FORMAT);
+	(void) fprintf (stderr, LOOKUP_C_BAD_FORMAT, MAYBE_CR (stderr));
 	return (-1);
 	}
     (void) close (hashfd);
@@ -243,22 +247,24 @@ int linit ()
 	    if (dp->word == (unsigned char *) -1)
 		dp->word = NULL;
 	    else
-		dp->word = &hashstrings [ (int)(dp->word) ];
+		dp->word = &hashstrings[(unsigned long) dp->word];
 	    if (dp->next == (struct dent *) -1)
 		dp->next = NULL;
 	    else
-		dp->next = &hashtbl [ (int)(dp->next) ];
+		dp->next = &hashtbl[(unsigned long) dp->next];
 	    }
 	}
 
     for (i = numsflags + numpflags, entry = sflaglist; --i >= 0; entry++)
 	{
 	if (entry->stripl)
-	    entry->strip = (ichar_t *) &hashstrings[(int) entry->strip];
+	    entry->strip =
+	      (ichar_t *) &hashstrings[(unsigned long) entry->strip];
 	else
 	    entry->strip = NULL;
 	if (entry->affl)
-	    entry->affix = (ichar_t *) &hashstrings[(int) entry->affix];
+	    entry->affix =
+	      (ichar_t *) &hashstrings[(unsigned long) entry->affix];
 	else
 	    entry->affix = NULL;
 	}
@@ -319,7 +325,8 @@ int linit ()
 		  sizeof (struct flagptr));
 	    if (ind->pu.fp == NULL)
 		{
-		(void) fprintf (stderr, LOOKUP_C_NO_LANG_SPACE);
+		(void) fprintf (stderr, LOOKUP_C_NO_LANG_SPACE,
+		  MAYBE_CR (stderr));
 		return (-1);
 		}
 	    ind->numents = 0;
@@ -381,7 +388,8 @@ int linit ()
 	        sizeof (struct flagptr));
 	    if (ind->pu.fp == NULL)
 		{
-		(void) fprintf (stderr, LOOKUP_C_NO_LANG_SPACE);
+		(void) fprintf (stderr, LOOKUP_C_NO_LANG_SPACE,
+		  MAYBE_CR (stderr));
 		return (-1);
 		}
 	    ind->numents = 0;
@@ -401,7 +409,7 @@ int linit ()
 	  malloc (hashheader.nstrchartype * sizeof (struct strchartype));
 	if (chartypes == NULL)
 	    {
-	    (void) fprintf (stderr, LOOKUP_C_NO_LANG_SPACE);
+	    (void) fprintf (stderr, LOOKUP_C_NO_LANG_SPACE, MAYBE_CR (stderr));
 	    return (-1);
 	    }
 	for (i = 0, nextchar = hashheader.strtypestart;
@@ -495,7 +503,7 @@ struct dent * lookup (s, dotree)
 	/* quick strcmp, but only for equality */
 	s1 = dp->word;
 	if (s1  &&  s1[0] == schar[0]
-	  &&  strcmp ((char *) s1 + 1, schar + 1) == 0)
+	  &&  strcmp ((char *) s1 + 1, (char *) schar + 1) == 0)
 	    return dp;
 	while (dp->flagfield & MOREVARIANTS)	/* Skip variations */
 	    dp = dp->next;
