@@ -1,10 +1,11 @@
 #ifndef lint
 static char Rcs_Id[] =
-	"$Id: makedent.c,v 1.55 2001/07/25 21:51:46 geoff Exp $";
+	"$Id: makedent.c,v 1.59 2005/04/20 23:16:32 geoff Exp $";
 #endif
 
 /*
- * Copyright 1988, 1989, 1992, 1993, 1999, 2001, Geoff Kuenning, Claremont, CA
+ * Copyright 1988, 1989, 1992, 1993, 1999, 2001, 2005, Geoff Kuenning,
+ * Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -20,16 +21,8 @@ static char Rcs_Id[] =
  *    such.  Binary redistributions based on modified source code
  *    must be clearly marked as modified versions in the documentation
  *    and/or other materials provided with the distribution.
- * 4. Any web site or other electronic service that offers ispell for
- *    download or other electronic transfer as a separate entity, in
- *    either source or binary form, must also include a prominent statement
- *    indicating that information about ispell can be obtained from the
- *    following Web site URL:
- *	http://fmg-www.cs.ucla.edu/geoff/ispell.html
- *    If the offering service supports hyperlinks, the aforementioned
- *    Web site must also be offered as a hyperlink.  Condition #4 does
- *    not apply if ispell is offered only as part of a larger, aggregated
- *    product such as a word processor or packaged operating system.
+ * 4. The code that causes the 'ispell -v' command to display a prominent
+ *    link to the official ispell Web site may not be removed.
  * 5. The name of Geoff Kuenning may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
@@ -49,6 +42,18 @@ static char Rcs_Id[] =
 
 /*
  * $Log: makedent.c,v $
+ * Revision 1.59  2005/04/20 23:16:32  geoff
+ * Rename some variables to make them more meaningful.
+ *
+ * Revision 1.58  2005/04/14 23:11:36  geoff
+ * Move initck from ispell.c.
+ *
+ * Revision 1.57  2005/04/14 15:19:37  geoff
+ * Stick in a couple of strategic const declarations.
+ *
+ * Revision 1.56  2001/09/06 00:30:28  geoff
+ * Many changes from Eli Zaretskii to support DJGPP compilation.
+ *
  * Revision 1.55  2001/07/25 21:51:46  geoff
  * Minor license update.
  *
@@ -104,6 +109,7 @@ static char Rcs_Id[] =
 #include "ispell.h"
 #include "proto.h"
 #include "msgs.h"
+#include <ctype.h>
 
 int		makedent P ((unsigned char * lbuf, int lbuflen,
 		  struct dent * ent));
@@ -128,10 +134,10 @@ static void	flagout P ((FILE * outfile, int flag));
 int		stringcharlen P ((unsigned char * bufp, int canonical));
 int		strtoichar P ((ichar_t * out, unsigned char * in, int outlen,
 		  int canonical));
-int		ichartostr P ((unsigned char * out, ichar_t * in, int outlen,
-		  int canonical));
+int		ichartostr P ((unsigned char * out, const ichar_t * in,
+		  int outlen, int canonical));
 ichar_t *	strtosichar P ((unsigned char * in, int canonical));
-unsigned char *	ichartosstr P ((ichar_t * in, int canonical));
+unsigned char *	ichartosstr P ((const ichar_t * in, int canonical));
 char *		printichar P ((int in));
 #ifndef ICHAR_IS_CHAR
 ichar_t *	icharcpy P ((ichar_t * out, ichar_t * in));
@@ -141,6 +147,7 @@ int		icharncmp P ((ichar_t * s1, ichar_t * s2, int n));
 #endif /* ICHAR_IS_CHAR */
 int		findfiletype P ((char * name, int searchnames,
 		  int * deformatter));
+void		initckch P ((const char * wchars));
 
 static int  	has_marker;
 
@@ -159,12 +166,12 @@ int makedent (lbuf, lbuflen, d)
     ichar_t *		ip;
     unsigned char *	p;
     int			bit;
-    unsigned int	len;
+    int			len;
 
-    /* Strip off any trailing newline */
-    len = strlen ((char *) lbuf) - 1;
-    if (lbuf[len] == '\n')
-	lbuf[len] = '\0';
+    /* Strip off any trailing newlines and returns */
+    len = (int) strlen ((char *) lbuf) - 1;
+    while (len >= 0  &&  (lbuf[len] == '\n'  ||  lbuf[len] == '\r'))
+	lbuf[len--] = '\0';
 
     d->next = NULL;
     /* WARNING:  flagfield might be the same as mask! See ispell.h. */
@@ -199,12 +206,12 @@ int makedent (lbuf, lbuflen, d)
 	      ||  ip == ibuf  ||  ip[1] == 0)
 		{
 		(void) fprintf (stderr, MAKEDENT_C_BAD_WORD_CHAR,
-		  (char *) lbuf);
+		  MAYBE_CR (stderr), (char *) lbuf, MAYBE_CR (stderr));
 		return -1;
 		}
 	    }
 	}
-    len = strlen ((char *) lbuf);
+    len = (int) strlen ((char *) lbuf);
     /*
     ** Figure out the capitalization rules from the capitalization of
     ** the sample entry.
@@ -220,7 +227,8 @@ int makedent (lbuf, lbuflen, d)
     d->word = mymalloc ((unsigned) len + 1);
     if (d->word == NULL)
 	{
-	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, (char *) lbuf);
+	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE,
+			MAYBE_CR (stderr), (char *) lbuf, MAYBE_CR (stderr));
 	return -1;
 	}
 
@@ -237,7 +245,8 @@ int makedent (lbuf, lbuflen, d)
 	if (bit >= 0  &&  bit <= LARGESTFLAG)
 	    SETMASKBIT (d->mask, bit);
 	else if (!aflag)
-	    (void) fprintf (stderr, BAD_FLAG, (unsigned char) *p);
+	    (void) fprintf (stderr, BAD_FLAG,
+	      MAYBE_CR (stderr), (unsigned char) *p, MAYBE_CR (stderr));
 	p++;
 	if (*p == hashheader.flagmarker)
 	    p++;		/* Handle old-format dictionaries too */
@@ -313,7 +322,8 @@ int addvheader (dp)
     tdent = (struct dent *) mymalloc (sizeof (struct dent));
     if (tdent == NULL)
 	{
-	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, (char *) dp->word);
+	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE,
+	  MAYBE_CR (stderr), (char *) dp->word, MAYBE_CR (stderr));
 	return -1;
 	}
     *tdent = *dp;
@@ -327,7 +337,7 @@ int addvheader (dp)
 	if (tdent->word == NULL)
 	    {
 	    (void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE,
-	      (char *) dp->word);
+	      MAYBE_CR (stderr), (char *) dp->word, MAYBE_CR (stderr));
 	    myfree ((char *) tdent);
 	    return -1;
 	    }
@@ -446,7 +456,7 @@ int combinecaps (hdrp, newp)
 	if (tdent == NULL)
 	    {
 	    (void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE,
-	      (char *) newp->word);
+	      MAYBE_CR (stderr), (char *) newp->word, MAYBE_CR (stderr));
 	    return -1;
 	    }
 	*tdent = *newp;
@@ -793,7 +803,7 @@ int stringcharlen (bufp, canonical)
 #endif /* SLOWMULTIPLY */
     lowstringno = 0;
     highstringno = hashheader.nstrchars - 1;
-    dupwanted = canonical ? 0 : defdupchar;
+    dupwanted = canonical ? 0 : defstringgroup;
     while (lowstringno <= highstringno)
 	{
 	stringno = (lowstringno + highstringno) >> 1;
@@ -815,7 +825,7 @@ int stringcharlen (bufp, canonical)
 	    }
 	if (*stringcur == '\0')
 	    {
-	    if (hashheader.dupnos[stringno] == dupwanted)
+	    if (hashheader.groupnos[stringno] == dupwanted)
 		{
 		/* We have a match */
 		laststringch = hashheader.stringdups[stringno];
@@ -833,7 +843,7 @@ int stringcharlen (bufp, canonical)
 	    highstringno = stringno - 1;
 	else if (*bufcur > *stringcur)
 	    lowstringno = stringno + 1;
-	else if (dupwanted < hashheader.dupnos[stringno])
+	else if (dupwanted < hashheader.groupnos[stringno])
 	    highstringno = stringno - 1;
 	else
 	    lowstringno = stringno + 1;
@@ -881,7 +891,8 @@ int strtoichar (out, in, outlen, canonical)
 int ichartostr (out, in, outlen, canonical)
     register unsigned char *
 			out;		/* Where to put result */
-    register ichar_t *	in;		/* String to convert */
+    register const ichar_t *
+			in;             /* String to convert */
     int			outlen;		/* Size of output buffer, bytes */
     int			canonical;	/* NZ for canonical form */
     {
@@ -902,7 +913,7 @@ int ichartostr (out, in, outlen, canonical)
 		{
 		for (i = hashheader.nstrchars;  --i >= 0;  )
 		    {
-		    if (hashheader.dupnos[i] == defdupchar
+		    if (hashheader.groupnos[i] == defstringgroup
 		      &&  hashheader.stringdups[i] == ch)
 			{
 			ch = i;
@@ -921,7 +932,7 @@ int ichartostr (out, in, outlen, canonical)
     }
 
 /*
- * Convert a string to an ichar_t, storing the result in a static area.
+ * Convert a string to an ichar_t *, storing the result in a static area.
  */
 ichar_t * strtosichar (in, canonical)
     unsigned char *	in;		/* String to convert */
@@ -935,10 +946,10 @@ ichar_t * strtosichar (in, canonical)
     }
 
 /*
- * Convert an ichar_t to a string, storing the result in a static area.
+ * Convert an ichar_t * to a string, storing the result in a static area.
  */
 unsigned char * ichartosstr (in, canonical)
-    ichar_t *		in;		/* Internal string to convert */
+    const ichar_t *	in;		/* Internal string to convert */
     int			canonical;	/* NZ for canonical conversion */
     {
     static unsigned char
@@ -1092,6 +1103,89 @@ int findfiletype (name, searchnames, deformatter)
     return -1;
     }
 
+void initckch (wchars)
+    const char *	wchars;		/* Characters in -w option, if any */
+    {
+    register ichar_t	c;
+    char		num[4];
+
+    for (c = 0; c < (ichar_t) (SET_SIZE + hashheader.nstrchars); ++c)
+	{
+	if (iswordch (c))
+	    {
+	    if (!mylower (c))
+		{
+		Try[Trynum] = c;
+		++Trynum;
+		}
+	    }
+	else if (isboundarych (c))
+	    {
+	    Try[Trynum] = c;
+	    ++Trynum;
+	    }
+	}
+    if (wchars != NULL)
+	{
+	while (Trynum < SET_SIZE + MAXSTRINGCHARS  &&  *wchars != '\0')
+	    {
+	    if (*wchars != 'n'  &&  *wchars != '\\')
+		{
+		c = *wchars;
+		++wchars;
+		}
+	    else
+		{
+		++wchars;
+		num[0] = '\0'; 
+		num[1] = '\0'; 
+		num[2] = '\0'; 
+		num[3] = '\0';
+		if (isdigit (wchars[0]))
+		    {
+		    num[0] = wchars[0];
+		    if (isdigit (wchars[1]))
+			{
+			num[1] = wchars[1];
+			if (isdigit (wchars[2]))
+			    num[2] = wchars[2];
+			}
+		    }
+		if (wchars[-1] == 'n')
+		    {
+		    wchars += strlen (num);
+		    c = atoi (num);
+		    }
+		else
+		    {
+		    wchars += strlen (num);
+		    c = 0;
+		    if (num[0])
+			c = num[0] - '0';
+		    if (num[1])
+			{
+			c <<= 3;
+			c += num[1] - '0';
+			}
+		    if (num[2])
+			{
+			c <<= 3;
+			c += num[2] - '0';
+			}
+		    }
+		}
+	    c &= 0xFF;
+	    if (!hashheader.wordchars[c])
+		{
+		hashheader.wordchars[c] = 1;
+		hashheader.sortorder[c] = hashheader.sortval++;
+		Try[Trynum] = c;
+		++Trynum;
+		}
+	    }
+	}
+    }
+
 /*
  * The following routines are all dummies for the benefit of lint.
  */
