@@ -1,7 +1,9 @@
+#ifndef PROTO_H_INCLUDED
+#define PROTO_H_INCLUDED
 /*
- * $Id: proto.h,v 1.24 2001/07/25 21:51:47 geoff Exp $
+ * $Id: proto.h,v 1.28 2005/04/26 22:40:08 geoff Exp $
  *
- * Copyright 1992, 1993, 1999, 2001, Geoff Kuenning, Claremont, CA
+ * Copyright 1992, 1993, 1999, 2001, 2005, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,16 +19,8 @@
  *    such.  Binary redistributions based on modified source code
  *    must be clearly marked as modified versions in the documentation
  *    and/or other materials provided with the distribution.
- * 4. Any web site or other electronic service that offers ispell for
- *    download or other electronic transfer as a separate entity, in
- *    either source or binary form, must also include a prominent statement
- *    indicating that information about ispell can be obtained from the
- *    following Web site URL:
- *	http://fmg-www.cs.ucla.edu/geoff/ispell.html
- *    If the offering service supports hyperlinks, the aforementioned
- *    Web site must also be offered as a hyperlink.  Condition #4 does
- *    not apply if ispell is offered only as part of a larger, aggregated
- *    product such as a word processor or packaged operating system.
+ * 4. The code that causes the 'ispell -v' command to display a prominent
+ *    link to the official ispell Web site may not be removed.
  * 5. The name of Geoff Kuenning may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
@@ -47,6 +41,19 @@
 
 /*
  * $Log: proto.h,v $
+ * Revision 1.28  2005/04/26 22:40:08  geoff
+ * Add double-inclusion protection.  Include ispell.h for the definition of P.
+ *
+ * Revision 1.27  2005/04/20 23:16:32  geoff
+ * Add inpossibilities.
+ *
+ * Revision 1.26  2005/04/14 23:11:36  geoff
+ * Prototype initckch.
+ *
+ * Revision 1.25  2005/04/14 14:38:23  geoff
+ * Update license.  Rename some functions to avoid collisions with
+ * libraries.  Add a couple of "const" declarations.
+ *
  * Revision 1.24  2001/07/25 21:51:47  geoff
  * Minor license update.
  *
@@ -108,6 +115,8 @@
  *
  */
 
+#include "ispell.h"		/* For definition of P */
+
 extern int	addvheader P ((struct dent * ent));
 extern void	askmode P ((void));
 extern void	backup P ((void));
@@ -127,7 +136,7 @@ extern void	correct P ((unsigned char * ctok, int ctokl, ichar_t * itok,
 extern char *	do_regex_lookup P ((char * expr, int whence));
 extern SIGNAL_TYPE done P ((int));
 extern void	dumpmode P ((void));
-extern void	erase P ((void));
+extern void	ierase P ((void));
 extern int	expand_pre P ((unsigned char * croot, ichar_t * rootword,
 		  MASKTYPE mask[], int option, unsigned char *extra));
 extern int	expand_suf P ((unsigned char * croot, ichar_t * rootword,
@@ -147,10 +156,11 @@ extern ichar_t * icharcpy P ((ichar_t * out, ichar_t * in));
 extern int	icharlen P ((ichar_t * str));
 extern int	icharncmp P ((ichar_t * s1, ichar_t * s2, int n));
 #endif /* ICHAR_IS_CHAR */
-extern int	ichartostr P ((unsigned char * out, ichar_t * in, int outlen,
-		  int canonical));
+extern int	ichartostr P ((unsigned char * out, const ichar_t * in,
+		  int outlen, int canonical));
 extern unsigned char *
-		ichartosstr P ((ichar_t * in, int canonical));
+		ichartosstr P ((const ichar_t * in, int canonical));
+extern void	initckch P ((const char * wchars));
 extern int	ins_root_cap P ((ichar_t * word, ichar_t * pattern,
 		  int prestrip, int preadd, int sufstrip, int sufadd,
 		  struct dent * firstdent, struct flagent * pfxent,
@@ -162,7 +172,8 @@ extern void	lowcase P ((ichar_t * string));
 extern int	makedent P ((unsigned char * lbuf, int lbuflen,
 		  struct dent * d));
 extern void	makepossibilities P ((ichar_t * word));
-extern void	move P ((int row, int col));
+extern int	inpossibilities P ((unsigned char * ctok));
+extern void	imove P ((int row, int col));
 extern void	normal P ((void));
 extern char *	printichar P ((int in));
 extern int	init_keyword_table P ((char * rawtags, char * envvar,
@@ -321,3 +332,5 @@ extern char *	tputs P ((const char * str, int pad, int (*func) (int ch)));
 #define P(x)	x
 #endif /* __STDC__ */
 #endif /* GENERATE_LIBRARY_PROTOS */
+
+#endif /* PROTO_H_INCLUDED */
